-- Statistics.lua 1.0
-- Copyright 2010 by Gregory Smith

-- This script is licensed under a Creative Commons Attribution 3.0
-- Unported License
--
-- http://creativecommons.org/licenses/by/3.0/

-- weights for each projectile type
ProjectileTypes["missile"]._weight = 1.0
ProjectileTypes["missile"]._exp = 1.0

ProjectileTypes["grenade"]._weight = 1.0
ProjectileTypes["grenade"]._exp = 1.0

ProjectileTypes["pistol bullet"]._weight = 1.0
ProjectileTypes["pistol bullet"]._exp = 1.0

ProjectileTypes["rifle bullet"]._weight = 4.0 / 52.0
ProjectileTypes["rifle bullet"]._exp = 0.5

ProjectileTypes["shotgun bullet"]._weight = 1.0
ProjectileTypes["shotgun bullet"]._exp = 1.0

ProjectileTypes["flamethrower burst"]._weight = 4.0 / 210.0
ProjectileTypes["flamethrower burst"]._exp = 0.5

ProjectileTypes["alien weapon"]._weight = 1.0
ProjectileTypes["alien weapon"]._exp = 1.0

ProjectileTypes["fusion bolt minor"]._weight = 1.0
ProjectileTypes["fusion bolt minor"]._exp = 1.0

ProjectileTypes["fusion bolt major"]._weight = 1.0
ProjectileTypes["fusion bolt major"]._exp = 1.0

ProjectileTypes["smg bullet"]._weight = 2.0 / 32.0
ProjectileTypes["smg bullet"]._exp = 0.5

-- do not edit below this line

Triggers = {}

function Triggers.init(resuming)
   if Game.version < "20100118" then
      -- silently exit
      Triggers = {}
      return
   end

   if not resuming or not Game.resume_saved then
      Game._level_start_tick = Game.ticks
      for p in Players() do 
	 p._damage_given = 0
	 p._damage_taken = 0
	 p._fired = {}
	 p._hit = {}
	 p._shots = {}
	 for t in ProjectileTypes() do
	    p._fired[t] = 0
	    p._hit[t] = 0
	 end

	 p._overhead_map = false
	 p._display_statistics = false
	 p._display_player_index = p.index
      end
   else
      for p in Players() do
	 p._display_player_index  = p._display_player_index % # Players
      end
   end
end

function projectile_hit(player, projectile, damage_amount)
   if projectile then
      if not player._shots[projectile._tick]._hit then
	 player._shots[projectile._tick]._hit = true
	 player._hit[projectile.type] = player._hit[projectile.type] + 1
      end
   end
   player._damage_given = player._damage_given + damage_amount
end

function Triggers.projectile_detonated()
   check_track_projectiles()
end

function Triggers.monster_damaged(_, aggressor_monster, _, damage_amount, projectile) 
   check_track_projectiles()
   if aggressor_monster and aggressor_monster.player then
      projectile_hit(aggressor_monster.player, projectile, damage_amount)
   end
end

function Triggers.player_damaged(victim, aggressor_player, aggressor_monster, _, damage_amount, projectile)
   check_track_projectiles()
   if aggressor_player then
      projectile_hit(aggressor_player, projectile, damage_amount)
   end

   victim._damage_taken = victim._damage_taken + damage_amount
end

function projectile_created(projectile)
   if projectile.owner and projectile.owner.player then
      local player = projectile.owner.player
      if not player._shots[Game.ticks] then
	 player._shots[Game.ticks] = {}
	 player._shots[Game.ticks]._count = 1
	 player._shots[Game.ticks]._hit = false
	 player._fired[projectile.type] = player._fired[projectile.type] + 1
      else
	 player._shots[Game.ticks]._count = player._shots[Game.ticks]._count + 1
      end
      projectile._tick = Game.ticks
   end
end

function projectile_destroyed(index, owner, ticks)
   if owner and owner.valid and owner.player then
      owner.player._shots[ticks]._count = owner.player._shots[ticks]._count - 1
      if owner.player._shots[ticks]._count == 0 then
	 owner.player._shots[ticks] = nil
      end
   end
end

tracked_projectiles = {}

function track_projectile(projectile)
   tracked_projectiles[projectile.index] = {}
   tracked_projectiles[projectile.index].tick = Game.ticks
   tracked_projectiles[projectile.index].owner = projectile.owner
   projectile_created(projectile)
end   

function track_projectiles()
   for p in Projectiles() do
      if not tracked_projectiles[p.index] then
	 track_projectile(p)
      end
   end

   for key, _ in pairs(tracked_projectiles) do
      if not Projectiles[key] then
	 local t = tracked_projectiles[key]
	 tracked_projectiles[key] = nil
	 projectile_destroyed(key, t.owner, t.tick)
      end
   end
end

last_track_projectiles = -1

function check_track_projectiles()
   if Game.ticks > last_track_projectiles then
      track_projectiles()
      last_track_projectiles = Game.ticks
   end
end

function display_instructions()
   Players.print("Statistics Lua enabled; controls available in overhead map")
end

function Triggers.idle()
   if Game.ticks == 0 then
      display_instructions()
   end

   track_projectiles()

   local lp
   for p in Players() do
      if p.action_flags.toggle_map then
	 p._overhead_map = not p._overhead_map
      end
      if p._overhead_map then
	 if p.action_flags.cycle_weapons_backward then
	    p._display_player_index = (p._display_player_index + # Players - 1) % # Players
	    p.action_flags.cycle_weapons_backward = false
	 end
	 if p.action_flags.cycle_weapons_forward then
	    p._display_player_index = (p._display_player_index + 1) % # Players
	    p.action_flags.cycle_weapons_forward = false
	 end
	 if p.action_flags.action_trigger then
	    p._display_statistics = not p._display_statistics
	    p.action_flags.action_trigger = false
	 end
      end
      if p.local_ then
	 lp = p
      end
      if Game.ticks % 30 == 0 then
	 local minutes = (Game.ticks - Game._level_start_tick) / 30 / 60
	 if minutes > 0 then
	    p._dpm = p._damage_given / minutes
	 else
	    p._dpm = 0
	 end
      end
   end

   if lp._overhead_map then
      lp.overlays[0].text = Players[lp._display_player_index].name
      lp.overlays[0]:fill_icon("white")
      if lp._display_statistics then
	 lp.overlays[1].text = "display on"
      else
	 lp.overlays[1].text = "display off"
      end
      lp.overlays[1].icon = icon_action
      lp.overlays[2].text = "next player"
      lp.overlays[2].icon = icon_next
      lp.overlays[3].text = "prev player"
      lp.overlays[3].icon = icon_prev
   elseif lp._display_statistics then
      local dp = Players[lp._display_player_index]
      local accuracy = 0
      local weight = 0
      for t in ProjectileTypes() do
	 if t._weight and dp._fired[t] > 0 then
	    weight = weight + t._weight * dp._fired[t]
	    accuracy = accuracy + t._weight * dp._fired[t] * ((dp._hit[t] / dp._fired[t]) ^ t._exp)
	 end
      end
      
      local total_accuracy = 1.0
      if weight > 0 then
	 total_accuracy = accuracy / weight
      end

      lp.overlays[0].text = Players[lp._display_player_index].name
      lp.overlays[0].icon = nil
      lp.overlays[1].text = string.format("%.1f", total_accuracy * 100) .. "% acc"
      lp.overlays[1].icon = nil
      lp.overlays[2].text = string.format("%.0f dmg/m", dp._dpm) 
      lp.overlays[2].icon = nil
      lp.overlays[3].text = dp._damage_taken .. " taken"
      lp.overlays[3].icon = nil
   else
      lp.overlays[0]:clear()
      lp.overlays[1]:clear()
      lp.overlays[2]:clear()
      lp.overlays[3]:clear()
   end
end

icon_action =
[[
3
 000000
.00FFFF00
+FFFFFF
.......  .......
...  . ++   ....
.. ++  ++ ++ ...
.. ++  ++ ++ . .
... ++ ++ ++  + 
... ++ ++ ++ ++ 
.  . +++++++ ++ 
 ++  ++++++++++ 
 +++ +++++++++ .
. ++++++++++++ .
.. +++++++++++ .
.. ++++++++++ ..
... +++++++++ ..
.... +++++++ ...
..... ++++++ ...
..... ++++++ ...
]]

icon_next =
[[
8
 000000
.DD0000
+777777
@FF6633
#00BB00
$888888
%AAAAAA
&FFFFFF00
&&&&&&&&@..@&&&&
&&&&&&&@.$$.@&&&
&&&&&&@.$$&%.$&&
&&&&&&.$$&&&.$&&
&&&&&&.$&&.....&
&&&&&&.$&&&...$$
&&&&&&.$&&&&.$$&
&&&&&&&$&&&&&$&&
&&&&&&&&&&&&&&&&
   &&    &&    &
 # +& ## +& ## +
 # +& ## +& ## +
 # +& ## +& ## +
 # +& ## +& ## +
   +&    +&    +
&+++&&++++&&++++
]]

icon_prev =
[[
8
 000000
.DD0000
+777777
@FF6633
#00BB00
$888888
%AAAAAA
&FFFFFF00
&&&@..@&&&&&&&&&
&&@.$$.$&&&&&&&&
&&.$$&%.$&&&&&&&
&&.$&&&%.%&&&&&&
.....&&&.$&&&&&&
&...$$&&.$&&&&&&
&&.$$&&&.$&&&&&&
&&&$&&&&&%&&&&&&
&&&&&&&&&&&&&&&&
    &&    &&   &
 ## +& ## +& # +
 ## +& ## +& # +
 ## +& ## +& # +
 ## +& ## +& # +
    +&    +&   +
&++++&&++++&&+++
]]



